@@ -613,6 +613,8 @@ typedef struct {
   char *id;
   char *version;
   char *default_executable;
+  char *env;
+  gsize env_size;
   GList *slices;
 } Bundle;
 
@@ -664,7 +666,7 @@ bundle_write (Bundle *bundle, GFile *dest, GError **error)
   gsize header_data_len;
   GlickBundleHeader *header;
   GlickSliceRef *slice_refs, *ref;
-  gsize id_offset, version_offset, exec_offset;
+  gsize id_offset, version_offset, exec_offset, env_offset;
   goffset offset, padding;
   char *exec;
   char pad[4096] = {0 };
@@ -682,7 +684,7 @@ bundle_write (Bundle *bundle, GFile *dest, GError **error)
   header_data_len =
     sizeof (GlickBundleHeader) +
     g_list_length (bundle->slices) * sizeof (GlickSliceRef) +
-    strlen (bundle->id) + strlen (bundle->version) + strlen (exec);
+    strlen (bundle->id) + strlen (bundle->version) + strlen (exec) + bundle->env_size;
   header_data = g_malloc0 (header_data_len);
   header = (GlickBundleHeader *)header_data;
   slice_refs = (GlickSliceRef *)(header_data + sizeof (GlickBundleHeader));
@@ -695,10 +697,13 @@ bundle_write (Bundle *bundle, GFile *dest, GError **error)
     g_list_length (bundle->slices) * sizeof (GlickSliceRef);
   version_offset = id_offset + strlen (bundle->id);
   exec_offset = version_offset + strlen (bundle->version);
+  env_offset = exec_offset + strlen (exec);
 
   memcpy (header_data + id_offset, bundle->id, strlen (bundle->id));
   memcpy (header_data + version_offset, bundle->version, strlen (bundle->version));
   memcpy (header_data + exec_offset, exec, strlen (exec));
+  if (bundle->env != NULL)
+    memcpy (header_data + env_offset, bundle->env, bundle->env_size);
   header->bundle_id_offset = GUINT32_TO_LE (id_offset);
   header->bundle_id_size = GUINT32_TO_LE (strlen (bundle->id));
   header->bundle_version_offset = GUINT32_TO_LE (version_offset);
@@ -708,6 +713,11 @@ bundle_write (Bundle *bundle, GFile *dest, GError **error)
       header->exec_offset = GUINT32_TO_LE (exec_offset);
       header->exec_size = GUINT32_TO_LE (strlen (exec));
     }
+  if (bundle->env != NULL)
+    {
+      header->env_offset = GUINT32_TO_LE (env_offset);
+      header->env_size = GUINT32_TO_LE (bundle->env_size);
+    }
   header->slices_offset = GUINT32_TO_LE (sizeof (GlickBundleHeader));
   header->num_slices = GUINT32_TO_LE (g_list_length (bundle->slices));
 
@@ -787,16 +797,11 @@ bundle_write (Bundle *bundle, GFile *dest, GError **error)
   return TRUE;
 }
 
-#define BUNDLE_GROUP_NAME "Bundle"
-#define BUNDLE_KEY_ID "Id"
-#define BUNDLE_KEY_VERSION "Version"
-#define BUNDLE_KEY_EXEC "Exec"
-#define BUNDLE_KEY_EXPORT "Export"
-
 static char *bundle_version;
 static char *bundle_id;
 static char *default_executable;
 static char **exports;
+static char **envs;
 
 static GOptionEntry entries[] =
 {
@@ -804,6 +809,7 @@ static GOptionEntry entries[] =
   { "bundle-version", 'v', 0, G_OPTION_ARG_STRING, &bundle_version, "Bundle Version (e.g 1.0)", "version" },
   { "default-executable", 'e', 0, G_OPTION_ARG_STRING, &default_executable, "Default executable path", "executable" },
   { "export", 'E', 0, G_OPTION_ARG_STRING_ARRAY, &exports, "Export file", "file" },
+  { "setenv", 's', 0, G_OPTION_ARG_STRING_ARRAY, &envs, "Set environmentvar", "value" },
   { NULL }
 };
 
@@ -851,6 +857,21 @@ main (int argc, char *argv[])
   if (default_executable)
     bundle_set_default_executable (bundle, default_executable);
 
+  if (envs != NULL)
+    {
+      GString *str;
+      int i;
+
+      str = g_string_new ("");
+      for (i = 0; envs[i] != NULL; i++)
+	{
+	  g_string_append (str, envs[i]);
+	  g_string_append_c (str, 0);
+	}
+      bundle->env_size = str->len;
+      bundle->env = g_string_free (str, FALSE);
+    }
+
   root = slurp_files (argv[1], "/", "/");
 
   exports_root = NULL;
